use crate::nodes::{Document, Nodes};
use crate::SUPPORTED_FORMAT_VERSION;

fn build_html(node: &Nodes) -> String {
    match node {
        Nodes::Literal(node) => {
            node.clone()
        }
        Nodes::Text(node) => {
            format!("<p>{}</p>", node.value.iter().map(build_html).collect::<String>())
        }
        Nodes::Link(node) => {
            format!("<a href=\"{}\">{}</a>", node.url, node.value.iter().map(build_html).collect::<String>())

        }
        Nodes::Image(node) => {
            format!("<img src=\"{}\" />", node.url)
        }
        Nodes::Header(node) => {
            format!("<h{}>{}</h{}>", node.level, node.value.iter().map(build_html).collect::<String>(), node.level)
        }
        Nodes::Divider => {
            "<hr />".to_string()
        }
        Nodes::Emphasis(node) => {
            format!("<em>{}</em>", node.value.iter().map(build_html).collect::<String>())
        }
        Nodes::Strong(node) => {
            format!("<strong>{}</strong>", node.value.iter().map(build_html).collect::<String>())
        }
    }
}

impl Document {
    pub fn to_html(&self) -> Result<String, anyhow::Error> {
        if self.version > SUPPORTED_FORMAT_VERSION {
            return Err(anyhow::anyhow!("Unsupported version"));
        }

        Ok(self.value.iter().map(build_html).collect::<String>())
    }
}