use anyhow::anyhow;
use markdown::mdast::Node;
use crate::nodes::{Document, Emphasis, Header, Image, Link, Nodes, Strong, Text};
use crate::SUPPORTED_FORMAT_VERSION;

fn do_parse(input: &Node) -> Nodes {
    let mut il_node: Nodes = match input {
        Node::Break(_) => {
            Nodes::Divider
        }
        Node::ThematicBreak(_) => {
            Nodes::Divider
        }
        Node::Image(node) => {
            let node = node.clone();
            
            Nodes::Image(Image {
                url: node.url,
                alt: Some(node.alt),
                blur_hash: None,
                width: None,
                height: None,
                value: vec![],
            })
        }
        Node::Link(node) => {
            Nodes::Link(Link {
                url: node.clone().url,
                value: vec![],
            })
        }
        Node::Text(node) => {
            Nodes::Literal(node.clone().value)
        }
        Node::Heading(node) => {
            Nodes::Header(Header {
                level: node.depth.to_string(),
                value: vec![],
            })
        }
        Node::Paragraph(_) => {
            Nodes::Text(Text {
                value: vec![],
            })
        }
        Node::Emphasis(_) => {
            Nodes::Emphasis(Emphasis {
                value: vec![],
            })
        }
        Node::Strong(_) => {
            Nodes::Strong(Strong {
                value: vec![],
            })
        }
        _ => {
             Nodes::Literal(input.to_string())
        }
    };

    if let Some(children) = input.children() {
        for i in children {
            match &mut il_node {
                Nodes::Text(node) => {
                    node.value.push(do_parse(i))
                }
                Nodes::Link(node) => {
                    node.value.push(do_parse(i))
                }
                Nodes::Header(node) => {
                    node.value.push(do_parse(i))
                }
                Nodes::Emphasis(node) => {
                    node.value.push(do_parse(i))
                }
                Nodes::Strong(node) => {
                    node.value.push(do_parse(i))
                }
                _ => {
                    eprintln!("Cannot have children?")
                }
            }
        }
    }

    il_node
}

pub fn from_markdown(value: &str) -> Result<Document, anyhow::Error> {
    let root = markdown::to_mdast(value, &markdown::ParseOptions::default()).expect("MDX?");

    let mut document = Document {
        version: SUPPORTED_FORMAT_VERSION,
        value: vec![],
    };

    for i in root.children().ok_or(anyhow!("Empty?"))? {
        document.value.push(do_parse(i));
    }

    Ok(document)
}

impl Document {
    pub fn to_markdown(&self) -> Result<Document, anyhow::Error> {
        todo!()
    }
}